import 'package:life_repository/entity/ipconfig_entity.dart';
import 'package:life_repository/entity/poi2_map_entity.dart';
import 'package:life_repository/entity/tip_map_entity.dart';

import 'impl/amap_repository_impl.dart';

///
/// TencentRepository
/// 高德地图相关的网络库
///
abstract class AMapRepository {
  ///
  /// 获取单例
  ///
  static AMapRepository get() {
    return AMapRepositoryImpl();
  }

  ///
  /// 输入提示是一类简单的HTTP接口，提供根据用户输入的关键词查询返回建议列表。
  /// [key]               请求服务权限标识(用户在高德地图官网申请Web服务API类型KEY)
  /// [keywords]          查询关键词
  /// [type]              POI分类(服务可支持传入多个分类，多个类型剑用“|”分隔、可选值：POI分类名称、分类代码、此处强烈建议使用分类代码，否则可能会得到不符合预期的结果)
  /// [location]          坐标(格式：“X,Y”（经度,纬度），不可以包含空格、建议使用location参数，可在此location附近优先返回搜索关键词信息、在请求参数city不为空时生效)
  /// [city]              搜索城市(可选值：citycode、adcode，不支持县级市。填入此参数后，会尽量优先返回此城市数据，但是不一定仅局限此城市，如：在深圳市搜天安门，返回北京天安门结果。)
  /// [citylimit]         仅返回指定城市数据（可选值：true/false） false
  /// [datatype]          返回的数据类型（多种数据类型用“|”分隔，可选值：all-返回所有数据类型、poi-返回POI数据类型、bus-返回公交站点数据类型、busline-返回公交线路数据类型）
  /// [sig]               数字签名（数字签名获取和使用方法：https://lbs.amap.com/faq/account/key/72）
  /// [output]            返回数据格式类型（可选值：JSON,XML）
  ///
  Future<List<MapTipEntity>> inputSearchTips({
    required String key,
    required String keywords,
    String? type,
    String? location,
    String? city,
    bool citylimit = false,
    String datatype = 'all',
    String? sig,
    String output = 'JSON',
  });

  ///
  /// 地点搜索服务2.0是一类Web API接口服务；服务提供多种场景的地点搜索能力，包括关键字搜索、周边搜索、多边形区域搜索、ID搜索。
  /// 周边搜索
  /// [key]               请求服务权限标识(用户在高德地图官网申请Web服务API类型KEY)
  /// [location]          中心点坐标（圆形区域检索中心点，不支持多个点。经度和纬度用","分割，经度在前，纬度在后，经纬度小数点后不得超过6位）
  /// [keywords]          地点关键字(需要被检索的地点文本信息。、多个关键字用“|”分割，文本总长度不可超过80字符)
  /// [types]             指定地点类型（点文本搜索接口支持按照设定的POI类型限定地点搜索结果；地点类型与poi typecode是同类内容，可以传入多个poi typecode，相互之间用“|”分隔，内容可以参考POI分类码表；地点（POI）列表的排序会按照高德搜索能力进行综合权重排序；120000（商务住宅）150000（交通设施服务））
  /// [radius]            搜索半径（取值范围:0-50000，大于50000时按默认值，单位：米）
  /// [sortrule]          排序规则（规定返回结果的排序规则。按距离排序：distance；综合排序：weight）
  /// [region]            搜索区划（增加指定区域内数据召回权重，如需严格限制召回数据在区域内，请搭配使用city_limit参数，可输入行政区划名或对应citycode或adcode） 无，默认全国范围内搜索
  /// [showFields]       返回结果控制（show_fields用来筛选response结果中可选字段。show_fields的使用需要遵循如下规则：1、具体可指定返回的字段类请见下方返回结果说明中的“show_fields”内字段类型；2、多个字段间采用“,”进行分割；3、show_fields未设置时，只返回基础信息类内字段。）
  /// [pageSize]         当前分页展示的数据条数(page_size的取值1-25)
  /// [pageNum]          请求第几分页(page_num的取值1-100)
  /// [sig]               数字签名（请参考数字签名获取和使用方法）
  /// [output]            返回结果格式类型（默认格式为json，目前只支持json格式；）
  ///
  Future<List<Poi2MapEntity>> poi2AroundSearch({
    required String key,
    required String location,
    String? keywords,
    String? types,
    String radius = '3000',
    String sortrule = 'distance',
    String? region,
    String? showFields,
    int pageSize = 10,
    int pageNum = 1,
    String? sig,
    String output = 'json',
  });

  ///
  /// IP定位是一套简单的HTTP接口，根据用户输入的IP地址，能够快速的帮用户定位IP的所在位置。
  /// IP定位：仅支持IPV4，不支持国外IP解析。
  /// [key]               请求服务权限标识
  /// [ip]                ip地址（需要搜索的IP地址（仅支持国内）、若用户不填写IP，则取客户http之中的请求来进行定位）
  /// [sig]               选择数字签名认证的付费用户必填
  ///
  Future<IpConfigEntity> ipConfigAddress({
    required String key,
    String? ip,
    String? sig,
  });

  Future<String?> addressToGps({
    required String key,
    String? address,
    String? city,
    String? sig,
  });

  ///
  /// 经纬度转地址
  /// [key]                   用户key
  /// [location]              经纬度坐标(116.481488,39.990464)
  ///
  Future<dynamic> geocodeAddress({String? key, String? location});
}
